//----------------------------------------------------------------
//
// OpenDCC
// Gateway 1.0
//
// Copyright (c) 2008-2009 Wolfgang Kufer
//
// This source file is subject of the GNU general public license 2,
// that is available at the world-wide-web at
// http://www.gnu.org/licenses/gpl.txt
//
//-----------------------------------------------------------------
//
// file:      hardware.h
// author:    Wolfgang Kufer
// contact:   kufer@gmx.de
// webpage:   http://www.opendcc.de
// history:   2008-12-08 V0.01 started
//
//-----------------------------------------------------------------
//
// purpose:   throtte for dcc
// content:   central definitions used in the project
//            all hardware project settings are done here!!
//            
//            1.   Prozessor, Timing
//                 (change here, if OpenDCC-Throttle is run on a different uP)
//            2.   IOs
//                 (change here, if OpenDCC-Throttle is run on a different board)
//
//-----------------------------------------------------------------
#ifndef __HARDWARE_H__
#define __HARDWARE_H__


//========================================================================
// 1. Processor Definitions
//========================================================================
//

// PROCESSOR: one of: __AVR_AT90can128__

// if changed: check timings, timer settings, baudrate

#if (__AVR_AT90CAN128__)
  // atmega32:   2kByte SRAM, 1kByte EEPROM
  #define SRAM_SIZE    4096
  #define EEPROM_SIZE  4096
  #define EEPROM_BASE  0x810000L
#else
  #warning: severe: no supported processor  
#endif
//
#ifndef F_CPU
// prevent compiler error by supplying a default 
# warning "F_CPU not defined for <config.h>", set default to 10MHz 
# define F_CPU 10000000UL
// if changed: check every place where it is used (possible range overflow in preprocessor!)
#endif

//------------------------------------------------------------------------
// Fuses of AT90can128 (just for documentation)
//------------------------------------------------------------------------
// Lock Bit Byte
//  7       6       5       4       3       2       1       0 
//  ------  ------  ------  ------  ------  ------  ------  ------
//  1       1       1       1       1       1       1       1       default, not locked
//
// Extended Fuse Byte:
//  7       6       5       4       3       2       1       0 
//  ------  ------  ------  ------  ------  ------  ------  ------
//                                  BOD2    BOD1    BOD0    TA0SEL  111=no BOD (*), 101=2,7V, 100=4,3V
//  1       1       1       1       0       1       0       1
//
// Fuse High Byte:
//  7       6       5       4       3       2       1       0 
//  ------  ------  ------  ------  ------  ------  ------  ------
//  OCDEN   JTAGEN  SPIEN   WDTON   EESAVE  BOOTSZ1 BOOTSZ0 BOOTRST
//  1       1       0       1       1       1       0       0
//  disabl  disabl  enabl   default no save =0xFC00         enter boot
//
// Fuse Low Byte:
//  7       6       5       4       3       2       1       0 
//  ------  ------  ------  ------  ------  ------  ------  ------
//  CKDIV8  CKOUT   SUT1    SUT0    CKSEL3  CKSEL2  CKSEL1  CKSEL0
//  1       1       0       0       1       1       1       0
//  no div  no clk  crystal, BOD ena, 10 MHz    




//========================================================================
// 2. Port Definitions
//========================================================================

// Port A
#define KEY_SCAN1   6     // out
#define KEY_SCAN0   7     // out

// Port B
#define SPI_CS_EE   0     // out
#define SPI_SCK  	1     // out  / runs dedicated
#define SPI_MOSI 	2     // out  / runs dedicated
#define SPI_MISO   	3     // in   / runs dedicated
#define LED_X0    	4     // out (spare)
#define LED_X1    	5     // out (spare)
#define LED_2     	6     // out (active low)
#define LED_1       7     // out (active low)

// Port C -> unused
#define KEY_0       0     // in
#define KEY_1       0     // in
#define KEY_2       0     // in
#define KEY_3       0     // in
#define KEY_4       0     // in
#define KEY_5       0     // in / shift
#define KEY_6       6     // in / knob
#define KEY_7       7     // in / emergency stop

// Port D
#define DCC_IN      0     // in
#define MY_RTS1     1     // in
#define MY_RXD1     2     // in
#define MY_TXD1     3     // out
#define MY_CTS1     4     // out
#define TXCAN       5     // out
#define RXCAN       6     // in
#define BOOTL       7     // in / -> this is used to enter bootloader

// Port E
#define XP_RXD      0     // in
#define XP_TXD      1     // out
#define XP_RE       2     // out
#define LED_XSTOP   3     // out (unused)
#define LED_YSTOP   4     // out (unused)
#define LCD_PWM     5     // out
#define CAN_DISABL  6     // out
#define XP_DE       7     // out

// Port F
#define VCCIO_SENSE 0     // in analog (VCCIO)
#define POWER_ON    1     // out (unused)
#define VINT        2     // in (unused)
#define A58         3     // in (reserved)
#define LED_Y0    	4     // out ((unused))
#define LED_Y1    	5     // out ((unused))
#define LED_Y2    	6     // out ((unused))
#define LED_Y3    	7     // out ((unused))

// Port G
#define XBEE_ENABL  0     // out (low = enabled)
#define XBEE_SLEEP  1     // out
#define SPI_ADR0    2     // out
#define FTDI_RESET  3     // out
#define SPI_CS_LCD  4     // out


//------------------------------------------------------------------------
// LED-Control and IO-Macros

#define LED_1_ON         PORTB &= ~(1<<LED_1); 
#define LED_1_OFF        PORTB |= (1<<LED_1);

#define LED_2_ON         PORTB &= ~(1<<LED_2); 
#define LED_2_OFF        PORTB |= (1<<LED_2);


#define MY_CTS_HIGH      PORTD |= (1<<MY_CTS1)
#define MY_CTS_LOW       PORTD &= ~(1<<MY_CTS1)
#define MY_CTS_STATE     (PORTD & (1<<MY_CTS1))

// RTS is aktive low
#define MY_RTS_DETECT    (!(PIND & (1<<MY_RTS1)))

#define SET_XBEE_TO_SLEEP   PORTG |= (1<<XBEE_SLEEP)

#define ENABLE_USB          PORTG |= (1<<XBEE_ENABL); PORTG |= (1<<FTDI_RESET)
#define ENABLE_XBEE         PORTG &= ~(1<<FTDI_RESET); PORTG &= ~(1<<XBEE_ENABL)
#define ENABLE_CAN          PORTE &= ~(1<<CAN_DISABL)
#define DISABLE_CAN         PORTE |= (1<<CAN_DISABL)
#define ENABLE_XP_RX        PORTE &= ~(1<<XP_RE)
#define ENABLE_XP_TX        PORTE &= ~(1<<XP_DE)
#define DISABLE_XP          PORTE |= (1<<XP_RE); PORTE |= (1<<XP_DE)

#define TURN_POWER_ON       PORTF |= (1<<POWER_ON)
#define TURN_POWER_OFF      PORTF &= ~(1<<POWER_ON)

// Calls

extern void init_hardware(void);

extern void advance_bootloader_led(void);  // tiny shifter

#endif   // hardware.h
